@@ -1,14 +1,15 @@
 use crate::csv::CsvEncoding;
 use crate::csv_core::csv::RunningSize;
 use crate::csv_core::parser::{drop_quotes, skip_whitespace};
-use arrow::array::{ArrayData, LargeStringArray};
-use polars_arrow::builder::BooleanBufferBuilder;
+use arrow::array::Utf8Array;
+use arrow::bitmap::MutableBitmap;
+use polars_arrow::prelude::FromDataUtf8;
 use polars_core::prelude::*;
 use std::fmt::Debug;
 
 trait ToPolarsError: Debug {
     fn to_polars_err(&self) -> PolarsError {
-        PolarsError::Other(
+        PolarsError::ComputeError(
             format!(
                 "Could not parse primitive type during csv parsing: {:?}.\
                 This can occur when a column was inferred as integer type but we stumbled upon a floating point value\
@@ -21,23 +22,22 @@ trait ToPolarsError: Debug {
 }
 
 impl ToPolarsError for lexical::Error {}
-impl ToPolarsError for fast_float::Error {}
 
-pub(crate) trait PrimitiveParser: ArrowPrimitiveType {
+pub(crate) trait PrimitiveParser: PolarsPrimitiveType {
     fn parse(bytes: &[u8]) -> Result<Self::Native>;
 }
 
 impl PrimitiveParser for Float32Type {
     #[inline]
     fn parse(bytes: &[u8]) -> Result<f32> {
-        let a = fast_float::parse(bytes).map_err(|e| e.to_polars_err())?;
+        let a = lexical::parse(bytes).map_err(|e| e.to_polars_err())?;
         Ok(a)
     }
 }
 impl PrimitiveParser for Float64Type {
     #[inline]
     fn parse(bytes: &[u8]) -> Result<f64> {
-        let a = fast_float::parse(bytes).map_err(|e| e.to_polars_err())?;
+        let a = lexical::parse(bytes).map_err(|e| e.to_polars_err())?;
         Ok(a)
     }
 }
@@ -120,7 +120,7 @@ pub(crate) struct Utf8Field {
     data: AlignedVec<u8>,
     // offsets in the string data buffer
     offsets: AlignedVec<i64>,
-    validity: BooleanBufferBuilder,
+    validity: MutableBitmap,
     rdr: csv_core::Reader,
 }
 
@@ -132,7 +132,7 @@ impl Utf8Field {
             name: name.to_string(),
             data: AlignedVec::with_capacity(str_capacity),
             offsets,
-            validity: BooleanBufferBuilder::new(capacity),
+            validity: MutableBitmap::with_capacity(capacity),
             rdr: csv_core::ReaderBuilder::new().delimiter(delimiter).build(),
         }
     }
@@ -151,10 +151,10 @@ impl ParsedBuffer<Utf8Type> for Utf8Field {
         // first check utf8 validity
         #[cfg(feature = "simdutf8")]
         let parse_result = simdutf8::basic::from_utf8(bytes)
-            .map_err(|_| PolarsError::Other("invalid utf8 data".into()));
+            .map_err(|_| PolarsError::ComputeError("invalid utf8 data".into()));
         #[cfg(not(feature = "simdutf8"))]
-        let parse_result =
-            std::str::from_utf8(bytes).map_err(|_| PolarsError::Other("invalid utf8 data".into()));
+        let parse_result = std::str::from_utf8(bytes)
+            .map_err(|_| PolarsError::ComputeError("invalid utf8 data".into()));
         let data_len = self.data.len();
 
         // check if field fits in the str data buffer
@@ -197,7 +197,7 @@ impl ParsedBuffer<Utf8Type> for Utf8Field {
                 // the n_written from csv-core are now valid bytes so we can update the length.
                 unsafe { self.data.set_len(data_len + n_written) }
                 self.offsets.push(self.data.len() as i64);
-                self.validity.append(true);
+                self.validity.push(true);
             }
             Err(err) => {
                 if matches!(encoding, CsvEncoding::LossyUtf8) {
@@ -208,11 +208,11 @@ impl ParsedBuffer<Utf8Type> for Utf8Field {
                     let b = s.as_bytes();
                     self.data.extend_from_slice(b);
                     self.offsets.push(self.data.len() as i64);
-                    self.validity.append(true);
+                    self.validity.push(true);
                 } else if ignore_errors {
                     // append null
                     self.offsets.push(self.data.len() as i64);
-                    self.validity.append(false);
+                    self.validity.push(false);
                 } else {
                     return Err(err);
                 }
@@ -240,7 +240,7 @@ impl ParsedBuffer<BooleanType> for BooleanChunkedBuilder {
         } else if ignore_errors || bytes.is_empty() {
             self.append_null();
         } else {
-            return Err(PolarsError::Other(
+            return Err(PolarsError::ComputeError(
                 format!(
                     "Error while parsing value {} at byte position {} as boolean",
                     start_pos,
@@ -305,7 +305,7 @@ pub(crate) fn init_buffers(
                     delimiter,
                 )),
                 other => {
-                    return Err(PolarsError::Other(
+                    return Err(PolarsError::ComputeError(
                         format!("Unsupported data type {:?} when reading a csv", other).into(),
                     ))
                 }
@@ -340,20 +340,20 @@ impl Buffer {
             Buffer::UInt64(v) => v.finish().into_series(),
             Buffer::Float32(v) => v.finish().into_series(),
             Buffer::Float64(v) => v.finish().into_series(),
-            Buffer::Utf8(mut v) => {
+            // Safety:
+            // We already checked utf8 validity during parsing
+            Buffer::Utf8(mut v) => unsafe {
                 v.offsets.shrink_to_fit();
                 v.data.shrink_to_fit();
-                let array_data = ArrayData::builder(ArrowDataType::LargeUtf8)
-                    .len(v.offsets.len() - 1)
-                    .add_buffer(v.offsets.into_arrow_buffer())
-                    .add_buffer(v.data.into_arrow_buffer())
-                    .null_bit_buffer(v.validity.finish())
-                    .build();
 
-                let arr = LargeStringArray::from(array_data);
+                let arr = Utf8Array::<i64>::from_data_unchecked_default(
+                    v.offsets.into(),
+                    v.data.into(),
+                    Some(v.validity.into()),
+                );
                 let ca = Utf8Chunked::new_from_chunks(&v.name, vec![Arc::new(arr)]);
                 ca.into_series()
-            }
+            },
         }
     }
 
@@ -369,7 +369,7 @@ impl Buffer {
             Buffer::Float64(v) => v.append_null(),
             Buffer::Utf8(v) => {
                 v.offsets.push(v.data.len() as i64);
-                v.validity.append(false);
+                v.validity.push(false);
             }
         };
     }
