@@ -1,22 +1,5 @@
 //! # (De)serialize JSON files.
 //!
-//! ## Write a DataFrame to a JSON file
-//!
-//! ## Example
-//!
-//! ```
-//! use polars_core::prelude::*;
-//! use polars_io::prelude::*;
-//! use std::fs::File;
-//!
-//! fn example(df: &mut DataFrame) -> Result<()> {
-//!     let mut file = File::create("example.csv").expect("could not create file");
-//!     
-//!     JsonWriter::new(&mut file)
-//!     .finish(df)
-//! }
-//! ```
-//!
 //! ## Read JSON to a DataFrame
 //!
 //! ## Example
@@ -77,32 +60,35 @@
 //! +-----+--------+-------+--------+
 //! ```
 //!
-use crate::finish_reader;
 use crate::prelude::*;
-use crate::utils::to_arrow_compatible_df;
-pub use arrow::json::LineDelimitedWriter;
-pub use arrow::json::ReaderBuilder;
+use crate::{finish_reader, ArrowReader};
+pub use arrow::{
+    error::Result as ArrowResult,
+    io::json::{Reader as ArrowJsonReader, ReaderBuilder},
+    record_batch::RecordBatch,
+};
 use polars_core::prelude::*;
-use std::io::{Read, Seek, Write};
+use std::io::Write;
+use std::io::{Read, Seek};
 use std::sync::Arc;
 
 // Write a DataFrame to JSON
-pub struct JsonWriter<'a, W: Write> {
+pub struct JsonWriter<W: Write> {
     /// File or Stream handler
-    buffer: &'a mut W,
+    buffer: W,
 }
 
-impl<'a, W> SerWriter<'a, W> for JsonWriter<'a, W>
+impl<W> SerWriter<W> for JsonWriter<W>
 where
     W: Write,
 {
-    fn new(buffer: &'a mut W) -> Self {
+    fn new(buffer: W) -> Self {
         JsonWriter { buffer }
     }
 
     fn finish(self, df: &DataFrame) -> Result<()> {
         let df = to_arrow_compatible_df(df);
-        let mut json_writer = LineDelimitedWriter::new(self.buffer);
+        let mut json_writer = arrow::io::json::LineDelimitedWriter::new(self.buffer);
 
         let batches = df.as_record_batches()?;
         json_writer.write_batches(&batches)?;
@@ -112,6 +98,16 @@ where
     }
 }
 
+impl<R: Read> ArrowReader for ArrowJsonReader<R> {
+    fn next_record_batch(&mut self) -> ArrowResult<Option<RecordBatch>> {
+        self.next()
+    }
+
+    fn schema(&self) -> Arc<Schema> {
+        Arc::new((&**self.schema()).into())
+    }
+}
+
 pub struct JsonReader<R>
 where
     R: Read + Seek,
@@ -185,25 +181,6 @@ mod test {
     use crate::prelude::*;
     use std::io::Cursor;
 
-    #[test]
-    fn write_json() {
-        let mut buf: Vec<u8> = Vec::new();
-        let mut df = create_df();
-
-        JsonWriter::new(&mut buf).finish(&mut df);
-
-        let json = std::str::from_utf8(&buf).unwrap();
-        assert_eq!(
-            r#"{"days":0,"temp":22.1}
-{"days":1,"temp":19.9}
-{"days":2,"temp":7.0}
-{"days":3,"temp":2.0}
-{"days":4,"temp":3.0}
-"#,
-            json
-        );
-    }
-
     #[test]
     fn read_json() {
         let basic_json = r#"{"a":1, "b":2.0, "c":false, "d":"4"}
